if [ -f /etc/a2ps-site.cfg ] && cmp -s /etc/defaults/etc/a2ps-site.cfg /etc/a2ps-site.cfg
then
    rm /etc/a2ps-site.cfg
fi

if [ -f /etc/a2ps.cfg ] && cmp -s /etc/defaults/etc/a2ps.cfg /etc/a2ps.cfg
then
    rm /etc/a2ps.cfg
fi

