if [ ! -f /etc/a2ps-site.cfg ]
then
    /usr/bin/mkdir -p /etc
    /usr/bin/cp /etc/defaults/etc/a2ps-site.cfg /etc/a2ps-site.cfg
fi

if [ ! -f /etc/a2ps.cfg ]
then
    /usr/bin/mkdir -p /etc
    /usr/bin/cp /etc/defaults/etc/a2ps.cfg /etc/a2ps.cfg
fi

cd /usr/share/a2ps/afm
./make_fonts_map.sh && /bin/mv -f fonts.map.new fonts.map
